@@ -28,12 +28,13 @@ __author__ = "Valentin Valls"
 __contact__ = "valentin.valls@esrf.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "16/10/2018"
+__date__ = "14/05/2019"
 __status__ = "production"
 
 import logging
 import os
 import sys
+import datetime
 from argparse import ArgumentParser
 
 logging.basicConfig(level=logging.INFO)
@@ -44,8 +45,8 @@ logger_uncaught = logging.getLogger("pyFAI-calib2.UNCAUGHT")
 import pyFAI.resources
 import pyFAI.calibrant
 import pyFAI.detectors
-
-import fabio
+import pyFAI.io.image
+from pyFAI.io.ponifile import PoniFile
 
 
 try:
@@ -65,13 +66,19 @@ def configure_parser_arguments(parser):
                         help="Filename where processed image is saved", metavar="FILE",
                         default=None)
     parser.add_argument("-v", "--verbose",
-                        action="store_true", dest="debug", default=False,
+                        dest="debug",
+                        action="store_true",
+                        default=False,
                         help="switch to debug/verbose mode")
     parser.add_argument('--debug',
                         dest="debug",
                         action="store_true",
                         default=False,
                         help='Set logging system in debug mode')
+    parser.add_argument("--opengl", "--gl", dest="opengl",
+                        action="store_true",
+                        default=False,
+                        help="Enable OpenGL rendering (else matplotlib is used)")
 
     # Settings
     parser.add_argument("-c", "--calibrant", dest="spacing", metavar="FILE",
@@ -91,34 +98,33 @@ def configure_parser_arguments(parser):
                         help="file containing the mask (for image reconstruction)", default=None)
     parser.add_argument("-p", "--pixel", dest="pixel",
                         help="size of the pixel in micron", default=None)
+    parser.add_argument("-s", "--spline", dest="spline",
+                        help="spline file describing the detector distortion", default=None)
+
+    parser.add_argument("-n", "--pt", dest="npt",
+                        help="file with datapoints saved. Example: basename.npt", default=None)
 
     # Not yet used
     parser.add_argument("-i", "--poni", dest="poni", metavar="FILE",
-                        help="file containing the diffraction parameter (poni-file).",
+                        help="file containing the diffraction parameter (poni-file) [not used].",
                         default=None)
     # Not yet used
     parser.add_argument("-b", "--background", dest="background",
-                        help="Automatic background subtraction if no value are provided",
+                        help="Automatic background subtraction if no value are provided [not used]",
                         default=None)
     # Not yet used
     parser.add_argument("-d", "--dark", dest="dark",
-                        help="list of comma separated dark images to average and subtract", default=None)
+                        help="list of comma separated dark images to average and subtract [not used]", default=None)
     # Not yet used
     parser.add_argument("-f", "--flat", dest="flat",
-                        help="list of comma separated flat images to average and divide", default=None)
-    # Not yet used
-    parser.add_argument("-s", "--spline", dest="spline",
-                        help="spline file describing the detector distortion", default=None)
-    # Not yet used
-    parser.add_argument("-n", "--pt", dest="npt",
-                        help="file with datapoints saved. Default: basename.npt", default=None)
+                        help="list of comma separated flat images to average and divide [not used]", default=None)
     # Not yet used
     parser.add_argument("--filter", dest="filter",
-                        help="select the filter, either mean(default), max or median",
+                        help="select the filter, either mean(default), max or median [not used]",
                         default=None)
 
     # Geometry
-    parser.add_argument("-l", "--distance", dest="distance", type=float,
+    parser.add_argument("-l", "--distance", dest="dist_mm", type=float,
                         help="sample-detector distance in millimeter. Default: 100mm", default=None)
     parser.add_argument("--dist", dest="dist", type=float,
                         help="sample-detector distance in meter. Default: 0.1m", default=None)
@@ -287,36 +293,15 @@ def displayExceptionBox(message, exc_info):
     :param Union[tuple,Exception] exc_info: An exception or the output of
         exc_info.
     """
-    logger.error(message, exc_info=True)
-
-    if isinstance(exc_info, BaseException):
-        exc_info = (type(exc_info), exc_info, exc_info.__traceback__)
-    elif not isinstance(exc_info, tuple):
-        exc_info = sys.exc_info()
-
-    if exc_info[2] is not None:
-        # Mimic the syntax of the default Python exception
-        import traceback
-        detailed = (''.join(traceback.format_tb(exc_info[2])))
-        detailed = '{1}\nTraceback (most recent call last):\n{2}{0}: {1}'.format(exc_info[0].__name__, exc_info[1], detailed)
-    else:
-        # There is no backtrace
-        detailed = '{0}: {1}'.format(exc_info[0].__name__, exc_info[1])
-
-    from silx.gui import qt
-    msg = qt.QMessageBox()
-    msg.setWindowTitle(message)
-    msg.setIcon(qt.QMessageBox.Critical)
-    msg.setInformativeText("%s" % exc_info[1])
-    msg.setDetailedText(detailed)
-
-    msg.raise_()
-    msg.exec_()
+    from pyFAI.gui.dialog import MessageBox
+    MessageBox.exception(None, message, exc_info, logger)
 
 
 def setup_model(model, options):
     """
     Setup the model using options from the command line.
+
+    :param pyFAI.gui.model.CalibrationModel model: Model of the application
     """
     args = options.args
 
@@ -325,9 +310,6 @@ def setup_model(model, options):
     # TODO: This should be removed
     import pyFAI.gui.cli_calibration
 
-    if options.debug:
-        logging.root.setLevel(logging.DEBUG)
-
     # Settings
     settings = model.experimentSettingsModel()
     if options.spacing:
@@ -358,7 +340,7 @@ def setup_model(model, options):
 
     if options.detector_name:
         try:
-            detector = pyFAI.gui.cli_calibration.get_detector(options.detector_name, args)
+            detector = pyFAI.gui.cli_calibration.get_detector(options.detector_name)
             if options.pixel:
                 logger.warning("Detector model already specified. Pixel size argument ignored.")
         except Exception as e:
@@ -381,13 +363,44 @@ def setup_model(model, options):
         except Exception as e:
             displayExceptionBox("Error while loading the spline file", e)
 
+    geometryFromPoni = None
+    if options.poni:
+        settings.poniFile().setValue(options.poni)
+        settings.poniFile().setSynchronized(True)
+
+        if os.path.exists(options.poni):
+            if detector is None:
+                poniFile = PoniFile()
+                poniFile.read_from_file(options.poni)
+                detector = poniFile.detector
+
+                from pyFAI.gui.model.GeometryModel import GeometryModel
+                geometryFromPoni = GeometryModel()
+                geometryFromPoni.distance().setValue(poniFile.dist)
+                geometryFromPoni.poni1().setValue(poniFile.poni1)
+                geometryFromPoni.poni2().setValue(poniFile.poni2)
+                geometryFromPoni.rotation1().setValue(poniFile.rot1)
+                geometryFromPoni.rotation2().setValue(poniFile.rot2)
+                geometryFromPoni.rotation3().setValue(poniFile.rot3)
+                geometryFromPoni.wavelength().setValue(poniFile.wavelength)
+
+                geometryHistory = model.geometryHistoryModel()
+                now = datetime.datetime.now()
+                geometryHistory.appendGeometry("Ponifile", now, geometryFromPoni, None)
+            else:
+                logger.warning("Detector redefined in the command line. Detector from --poni argument ignored.")
+        else:
+            logger.warning("PONI file '%s' do not exists. --poni option ignored.", options.poni)
+
     settings.detectorModel().setDetector(detector)
 
     if options.mask:
         try:
-            settings.maskFile().setValue(options.mask)
-            with fabio.open(options.mask) as mask:
-                settings.mask().setValue(mask.data)
+            with settings.mask().lockContext() as image_model:
+                image_model.setFilename(options.mask)
+                data = pyFAI.io.image.read_image_data(options.mask)
+                image_model.setValue(data)
+                image_model.setSynchronized(True)
         except Exception as e:
             displayExceptionBox("Error while loading the mask", e)
 
@@ -396,9 +409,11 @@ def setup_model(model, options):
     elif len(args) == 1:
         image_file = args[0]
         try:
-            settings.imageFile().setValue(image_file)
-            with fabio.open(image_file) as image:
-                settings.image().setValue(image.data)
+            with settings.image().lockContext() as image_model:
+                image_model.setFilename(image_file)
+                data = pyFAI.io.image.read_image_data(image_file)
+                image_model.setValue(data)
+                image_model.setSynchronized(True)
         except Exception as e:
             displayExceptionBox("Error while loading the image", e)
     else:
@@ -407,20 +422,29 @@ def setup_model(model, options):
     # Geometry
     # FIXME it will not be used cause the fitted geometry will be overwrited
     geometry = model.fittedGeometry()
-    if options.distance:
-        geometry.distance().setValue(1e-3 * options.distance)
-    if options.dist:
-        geometry.distance().setValue(options.dist)
-    if options.dist:
-        geometry.poni1().setValue(options.poni1)
-    if options.dist:
-        geometry.poni2().setValue(options.poni2)
-    if options.dist:
-        geometry.rotation1().setValue(options.rot1)
-    if options.dist:
-        geometry.rotation2().setValue(options.rot2)
-    if options.dist:
-        geometry.rotation3().setValue(options.rot3)
+    if (options.dist_mm is not None or options.dist is not None or
+            options.poni1 is not None or options.poni2 is not None or
+            options.rot1 is not None or options.rot2 is not None or
+            options.rot3 is not None):
+        if geometryFromPoni is not None:
+            logger.warning("Geometry redefined in the command line. Geometry from --poni argument was stored in the history.")
+        if options.dist_mm is not None:
+            geometry.distance().setValue(1e-3 * options.dist_mm)
+        if options.dist is not None:
+            geometry.distance().setValue(options.dist)
+        if options.poni1 is not None:
+            geometry.poni1().setValue(options.poni1)
+        if options.poni2 is not None:
+            geometry.poni2().setValue(options.poni2)
+        if options.rot1 is not None:
+            geometry.rotation1().setValue(options.rot1)
+        if options.rot2 is not None:
+            geometry.rotation2().setValue(options.rot2)
+        if options.rot3 is not None:
+            geometry.rotation3().setValue(options.rot3)
+    else:
+        if geometryFromPoni is not None:
+            geometry.setFrom(geometryFromPoni)
 
     # Constraints
     constraints = model.geometryConstraintsModel()
@@ -465,8 +489,6 @@ def setup_model(model, options):
 
     if options.outfile:
         logger.error("outfile option not supported")
-    if options.debug:
-        logger.error("debug option not supported")
 
     if options.reconstruct:
         logger.error("reconstruct option not supported")
@@ -477,9 +499,6 @@ def setup_model(model, options):
     if options.pixel:
         logger.error("pixel option not supported")
 
-    # FIXME poni file should be supported
-    if options.poni:
-        logger.error("poni option not supported")
     if options.background:
         logger.error("background option not supported")
     if options.dark:
@@ -501,6 +520,16 @@ def setup_model(model, options):
     if options.interactive is not True:
         logger.error("interactive option not supported")
 
+    if options.npt:
+        try:
+            from pyFAI.gui.helper import model_transform
+            from pyFAI.control_points import ControlPoints
+            controlPoints = ControlPoints(filename=options.npt)
+            peakSelectionModel = model.peakSelectionModel()
+            model_transform.initPeaksFromControlPoints(peakSelectionModel, controlPoints)
+        except Exception as e:
+            displayExceptionBox("Error while loading control-point file", e)
+
 
 def logUncaughtExceptions(exceptionClass, exception, stack):
     try:
@@ -526,12 +555,19 @@ def main():
     # --help must also work without Qt
     options = parse_options()
 
+    if options.debug:
+        logging.root.setLevel(logging.DEBUG)
+
     # Then we can load Qt
+    import silx
     from silx.gui import qt
+    if options.opengl:
+        silx.config.DEFAULT_PLOT_BACKEND = "opengl"
+
     # Make sure matplotlib is loaded first by silx
     import silx.gui.plot.matplotlib
-    from pyFAI.gui.calibration.CalibrationWindow import CalibrationWindow
-    from pyFAI.gui.calibration.CalibrationContext import CalibrationContext
+    from pyFAI.gui.CalibrationWindow import CalibrationWindow
+    from pyFAI.gui.CalibrationContext import CalibrationContext
 
     sys.excepthook = logUncaughtExceptions
     if options.qtargs is None:
