@@ -35,7 +35,7 @@ __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "22/08/2018"
+__date__ = "16/05/2019"
 __status__ = "stable"
 
 
@@ -53,6 +53,7 @@ from .. import utils
 from .. import average
 from ..utils import binning, expand2d, crc32
 from ..third_party.six import with_metaclass
+from ..utils.decorators import deprecated
 
 logger = logging.getLogger(__name__)
 
@@ -231,7 +232,12 @@ class Detector(with_metaclass(DetectorMeta, object)):
             (self.name, self.splineFile, self._pixel1, self._pixel2)
 
     def __copy__(self):
-        ":return: a shallow copy of itself"
+        """
+        Copy this detector.
+
+        :rtype: Detector
+        :return: A copy of this detector
+        """
         unmutable = ['_pixel1', '_pixel2', 'max_shape', 'shape', '_binning',
                      '_mask_crc', '_maskfile', "_splineFile", "_flatfield_crc",
                      "_darkcurrent_crc", "flatfiles", "darkfiles"]
@@ -244,7 +250,12 @@ class Detector(with_metaclass(DetectorMeta, object)):
         return new
 
     def __deepcopy__(self, memo=None):
-        ":return: a deep copy of itself"
+        """
+        Copy this detector all of the sub referenced objects.
+
+        :rtype: Detector
+        :return: A copy of this detector
+        """
         unmutable = ['_pixel1', '_pixel2', 'max_shape', 'shape', '_binning',
                      '_mask_crc', '_maskfile', "_splineFile", "_flatfield_crc",
                      "_darkcurrent_crc", "flatfiles", "darkfiles"]
@@ -559,14 +570,14 @@ class Detector(with_metaclass(DetectorMeta, object)):
                 delta1 = d1 - i1
                 delta2 = d2 - i2
                 pixels = self._pixel_corners[i1, i2]
-                A1 = pixels[:, :, 0, 1]
-                A2 = pixels[:, :, 0, 2]
-                B1 = pixels[:, :, 1, 1]
-                B2 = pixels[:, :, 1, 2]
-                C1 = pixels[:, :, 2, 1]
-                C2 = pixels[:, :, 2, 2]
-                D1 = pixels[:, :, 3, 1]
-                D2 = pixels[:, :, 3, 2]
+                A1 = pixels[..., 0, 1]
+                A2 = pixels[..., 0, 2]
+                B1 = pixels[..., 1, 1]
+                B2 = pixels[..., 1, 2]
+                C1 = pixels[..., 2, 1]
+                C2 = pixels[..., 2, 2]
+                D1 = pixels[..., 3, 1]
+                D2 = pixels[..., 3, 2]
                 # points A and D are on the same dim1 (Y), they differ in dim2 (X)
                 # points B and C are on the same dim1 (Y), they differ in dim2 (X)
                 # points A and B are on the same dim2 (X), they differ in dim1 (Y)
@@ -581,10 +592,10 @@ class Detector(with_metaclass(DetectorMeta, object)):
                     + C2 * delta1 * delta2 \
                     + D2 * (1.0 - delta1) * delta2
                 if not self.IS_FLAT:
-                    A0 = pixels[:, :, 0, 0]
-                    B0 = pixels[:, :, 1, 0]
-                    C0 = pixels[:, :, 2, 0]
-                    D0 = pixels[:, :, 3, 0]
+                    A0 = pixels[..., 0, 0]
+                    B0 = pixels[..., 1, 0]
+                    C0 = pixels[..., 2, 0]
+                    D0 = pixels[..., 3, 0]
                     p3 = A0 * (1.0 - delta1) * (1.0 - delta2) \
                         + B0 * delta1 * (1.0 - delta2) \
                         + C0 * delta1 * delta2 \
@@ -658,6 +669,32 @@ class Detector(with_metaclass(DetectorMeta, object)):
                         self._pixel_corners[:, :, 3, 0] = p3[:-1, 1:]
         return self._pixel_corners
 
+    def set_pixel_corners(self, ary):
+        """Sets the position of pixel corners with some additional validation
+
+        :param ary: This a 4D array which contains: number of lines,
+                                                    number of columns,
+                                                    corner index,
+                                                    position in space Z, Y, X
+        """
+        if ary is None:
+            # Leave as it is ... just reset the array
+            self._pixel_corners = None
+        else:
+            ary = numpy.ascontiguousarray(ary, dtype=numpy.float32)
+            # Validation for the array
+            assert ary.ndim == 4
+            assert ary.shape[3] == 3  # 3 coordinates in Z Y X
+            assert ary.shape[2] >= 3  # at least 3 corners per pixel
+
+            z = ary[..., 0]
+            is_flat = (z.max() == z.min() == 0.0)
+            with self._sem:
+                self.IS_CONTIGUOUS = False
+                self.IS_FLAT = is_flat
+                self.uniform_pixel = False  # This enforces the usage of pixel_corners
+                self._pixel_corners = ary
+
     def save(self, filename):
         """
         Saves the detector description into a NeXus file, adapted from:
@@ -874,6 +911,7 @@ class Detector(with_metaclass(DetectorMeta, object)):
 
     flatfield = property(get_flatfield, set_flatfield)
 
+    @deprecated(reason="Not maintained", since_version="0.17")
     def set_flatfiles(self, files, method="mean"):
         """
         :param files: file(s) used to compute the flat-field.
@@ -911,6 +949,7 @@ class Detector(with_metaclass(DetectorMeta, object)):
 
     darkcurrent = property(get_darkcurrent, set_darkcurrent)
 
+    @deprecated(reason="Not maintained", since_version="0.17")
     def set_darkfiles(self, files=None, method="mean"):
         """
         :param files: file(s) used to compute the dark.
@@ -966,6 +1005,27 @@ class NexusDetector(Detector):
     """
     Class representing a 2D detector loaded from a NeXus file
     """
+
+    _ATTRIBUTES_TO_CLONE = [
+        "aliases",
+        "IS_FLAT",
+        "IS_CONTIGUOUS",
+        "flatfield",
+        "darkcurrent",
+        "force_pixel",
+        "_binning",
+        "_pixel1",
+        "_pixel2",
+        "_pixel_corners",
+        "_binning",
+        "_filename",
+        "shape",
+        "mask",
+        "uniform_pixel",
+        "shape",
+        "max_shape",
+    ]
+
     def __init__(self, filename=None):
         Detector.__init__(self)
         self.uniform_pixel = True
@@ -994,26 +1054,26 @@ class NexusDetector(Detector):
             name = posixpath.split(det_grp.name)[-1]
             self.aliases = [name.replace("_", " "), det_grp.name]
             if "IS_FLAT" in det_grp:
-                self.IS_FLAT = det_grp["IS_FLAT"].value
+                self.IS_FLAT = det_grp["IS_FLAT"][()]
             if "IS_CONTIGUOUS" in det_grp:
-                self.IS_CONTIGUOUS = det_grp["IS_CONTIGUOUS"].value
+                self.IS_CONTIGUOUS = det_grp["IS_CONTIGUOUS"][()]
             if "flatfield" in det_grp:
-                self.flatfield = det_grp["flatfield"].value
+                self.flatfield = det_grp["flatfield"][()]
             if "darkcurrent" in det_grp:
-                self.darkcurrent = det_grp["darkcurrent"].value
+                self.darkcurrent = det_grp["darkcurrent"][()]
             if "force_pixel" in det_grp:
-                self.force_pixel = det_grp["force_pixel"].value
+                self.force_pixel = det_grp["force_pixel"][()]
             if "binning" in det_grp:
-                self._binning = tuple(i for i in det_grp["binning"].value)
+                self._binning = tuple(i for i in det_grp["binning"][()])
             if "pixel_size" in det_grp:
-                self._pixel1, self._pixel2 = det_grp["pixel_size"].value
+                self._pixel1, self._pixel2 = det_grp["pixel_size"][()]
             for what in ("max_shape", "shape"):
                 if what in det_grp:
-                    self.__setattr__(what, tuple(i for i in det_grp[what].value))
+                    self.__setattr__(what, tuple(i for i in det_grp[what][()]))
             if "mask" in det_grp:
-                self.mask = det_grp["mask"].value
+                self.mask = det_grp["mask"][()]
             if "pixel_corners" in det_grp:
-                self._pixel_corners = det_grp["pixel_corners"].value
+                self._pixel_corners = det_grp["pixel_corners"][()]
                 self.uniform_pixel = False
             else:
                 self.uniform_pixel = True
@@ -1043,6 +1103,26 @@ class NexusDetector(Detector):
 
     filename = property(get_filename)
 
+    def __copy__(self):
+        cloned = self.__class__()
+        for name in self._ATTRIBUTES_TO_CLONE:
+            if hasattr(self, name):
+                value = getattr(self, name)
+                setattr(cloned, name, value)
+        return cloned
+
+    def __deepcopy__(self, memo=None):
+        import copy
+        cloned = self.__class__()
+        if memo is not None:
+            memo[id(self)] = cloned
+        for name in self._ATTRIBUTES_TO_CLONE:
+            if hasattr(self, name):
+                value = getattr(self, name)
+                value = copy.deepcopy(value, memo)
+                setattr(cloned, name, value)
+        return cloned
+
     @classmethod
     def sload(cls, filename):
         """
