@@ -36,7 +36,7 @@ __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "16/10/2018"
+__date__ = "18/01/2019"
 __status__ = "production"
 
 import functools
@@ -535,18 +535,17 @@ class Cirpad(ImXPadS10):
 
     def _passage(self, corners, rot):
         shape = corners.shape
-        origine = corners[0][0][0, :]
+        deltaX, deltaY = 0.0, 0.0
         nmd = self._rotation(corners, rot)
-        u = corners[shape[0] - 1][0][1, :] - corners[0][0][0, :]
-        u = u / numpy.linalg.norm(u)
-        s = self._rotation(u, rot)
-        s = s / numpy.linalg.norm(s)
-        v = numpy.array([-u[1], u[0], u[2]])
-        r = origine - nmd[0][0][0, :]
-        w = (0.1e-3 + 0.24e-3 + 75.14e-3) * u + (0.8e-3) * v + (0.55e-3) * s + r
-        return self._translation(nmd, w)
+        # Size in mm of the chip in the Y direction (including 10px gap)
+        size_Y = ((560.0 + 3 * 6 + 20)*0.13/1000)
+        for i in range(1, int(round(numpy.abs(rot[2])/6.74))):
+            deltaX = deltaX + numpy.sin(numpy.deg2rad(-rot[2] -6.74*(i)))
+        for i in range(int(round(numpy.abs(rot[2])/6.74))):
+            deltaY = deltaY + numpy.cos(numpy.deg2rad(-rot[2] - 6.74*(i+1)))
+        return self._translation(nmd, [size_Y*deltaX,size_Y*deltaY, 0])
 
-    def get_pixel_corners(self):
+    def _get_pixel_corners(self):
         pixel_size1 = self._calc_pixels_size(self.MEDIUM_MODULE_SIZE[0],
                                              self.MODULE_SIZE[0],
                                              self.PIXEL_SIZE[0])
@@ -581,16 +580,19 @@ class Cirpad(ImXPadS10):
         corners[:, :, 3, 1] = pixel_center1 - pixel_size1 / 2.0
         corners[:, :, 3, 2] = pixel_center2 + pixel_size2 / 2.0
 
-        n_corners = corners
+        modules = [self._passage(corners, [self.ROT[0], self.ROT[1], self.ROT[2] * i]) for i in range(20)]
+        result = numpy.concatenate(modules, axis=0)
+        result = numpy.ascontiguousarray(result, result.dtype)
+        return result
 
-        # then we compute the positions of the 19 remaining ones
-        for _ in range(1, 20):
-            n_corners = self._passage(n_corners, self.ROT)
-            # Depending on the expected layout
-            corners = numpy.concatenate((corners, n_corners), axis=0)
-        return corners
+    def get_pixel_corners(self):
+        if self._pixel_corners is None:
+            with self._sem:
+                if self._pixel_corners is None:
+                    self._pixel_corners = self._get_pixel_corners()
+        return self._pixel_corners
 
-    # Pas fait encore
+    # TODO !!!
     def calc_cartesian_positions(self, d1=None, d2=None, center=True, use_cython=True):
         if (d1 is None) or d2 is None:
             d1 = mathutil.expand2d(numpy.arange(self.MAX_SHAPE[0]).astype(numpy.float32), self.MAX_SHAPE[1], False)
