@@ -31,6 +31,9 @@
 * calculating the geometry, i.e. the position in the detector space of each pixel of the detector
 * manages caches to store intermediate results
 
+NOTA: The Geometry class is not a "transformation class" which would take a
+detector and transform it. It is rather a description of the experimental setup.
+
 """
 
 from __future__ import division, print_function
@@ -39,17 +42,15 @@ __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "29/08/2018"
+__date__ = "16/05/2019"
 __status__ = "production"
 __docformat__ = 'restructuredtext'
 
 import logging
-from numpy import radians, degrees, arccos, arctan2, sin, cos, sqrt, pi
+from numpy import radians, degrees, arccos, arctan2, sin, cos, sqrt
 import numpy
 import os
 import threading
-import time
-import json
 from collections import namedtuple, OrderedDict
 
 from . import detectors
@@ -57,6 +58,7 @@ from . import units
 from .utils.decorators import deprecated
 from .utils import crc32
 from . import utils
+from .io import ponifile
 
 logger = logging.getLogger(__name__)
 
@@ -78,9 +80,13 @@ PolarizationDescription = namedtuple("PolarizationDescription",
 
 
 class Geometry(object):
-    """
-    This class is an azimuthal integrator based on P. Boesecke's geometry and
-    histogram algorithm by Manolo S. del Rio and V.A Sole
+    """This class is the parent-class of azimuthal integrator.
+
+    This class contains a detector (using composition) which provides the
+    position of all pixels, or only a limited set of pixel indices.
+    The Geometry class is responsible for translating/rotating those pixel to
+    their position in reference to the sample position.
+    The description of the experimental setup is inspired by the work of P. Boesecke
 
     Detector is assumed to be corrected from "raster orientation" effect.
     It is not addressed here but rather in the Detector object or at read time.
@@ -195,6 +201,19 @@ class Geometry(object):
                            f2d["tilt"], f2d["tiltPlanRotation"]))
         return os.linesep.join(lstTxt)
 
+    def check_chi_disc(self, range):
+        """Check the position of the :math:`\\chi` discontinuity
+
+        :param range: range of chi for the integration
+        :return: True if there is a problem
+        """
+        lower = range[0]
+        upper = range[-1]
+        disc = numpy.pi if self.chiDiscAtPi else 0
+        if (lower < disc) and (upper > disc):
+            logger.warning("Chi discontinuity in azimuthal range ! disc=%s in [%s, %s]", disc, lower, upper)
+            return 1
+
     def _calc_cartesian_positions(self, d1, d2, poni1=None, poni2=None):
         """
         Calculate the position in cartesian coordinate (centered on the PONI)
@@ -979,7 +998,7 @@ class Geometry(object):
         """
         Calculate the incidence angle (alpha) for current pixels (P).
         The poni being the point of normal incidence,
-        it's incidence angle is $\{alpha} = 0$ hence $cos(\{alpha}) = 1$
+        it's incidence angle is :math:`\\{alpha} = 0` hence :math:`cos(\\{alpha}) = 1`.
 
         :param d1: 1d or 2d set of points in pixel coord
         :param d2:  1d or 2d set of points in pixel coord
@@ -1032,7 +1051,7 @@ class Geometry(object):
         .. math::
 
             dOmega = \\frac{Omega(P)}{Omega(C)}
-                   = \\frac{A \cdot cos(a)}{SP^2} \cdot \\frac{SC^2}{A \cdot cos(0)}
+                   = \\frac{A \\cdot cos(a)}{SP^2} \\cdot \\frac{SC^2}{A \\cdot cos(0)}
                    = \\frac{3}{cos(a)}
                    = \\frac{SC^3}{SP^3}
 
@@ -1079,6 +1098,7 @@ class Geometry(object):
         SA = pix1*pix2/dist^2 * cos(incidence)^3
 
         """
+        shape = self.get_shape(shape)
         if order is True:
             self._dssa_order = 3.0
         else:
@@ -1100,71 +1120,48 @@ class Geometry(object):
     def get_config(self):
         """
         return the configuration as a dictionnary
-        
+
         :return: dictionary with the current configuration
         """
-
-        config = OrderedDict([("poni_version", 2)])
         with self._sem:
-            config["detector"] = self.detector.__class__.__name__
-            config["detector_config"] = self.detector.get_config()
-            config["dist"] = self._dist
-            config["poni1"] = self._poni1
-            config["poni2"] = self._poni2
-            config["rot1"] = self._rot1
-            config["rot2"] = self._rot2
-            config["rot3"] = self._rot3
-            if self._wavelength:
-                config["wavelength"] = self._wavelength
-        return config
+            # TODO: ponifile should not be used here
+            #     if it was only used for IO, it would be better to remove
+            #     this function
+            poni = ponifile.PoniFile(data=self)
+            return poni.as_dict()
+
+    def _init_from_poni(self, poni):
+        """Init the geometry from a poni object."""
+        if poni.detector is not None:
+            self.detector = poni.detector
+        if poni.dist is not None:
+            self._dist = poni.dist
+        if poni.poni1 is not None:
+            self._poni1 = poni.poni1
+        if poni.poni2 is not None:
+            self._poni2 = poni.poni2
+        if poni.rot1 is not None:
+            self._rot1 = poni.rot1
+        if poni.rot2 is not None:
+            self._rot2 = poni.rot2
+        if poni.rot3 is not None:
+            self._rot3 = poni.rot3
+        if poni.rot3 is not None:
+            self._wavelength = poni.wavelength
+        self.reset()
 
     def set_config(self, config):
         """
         Set the config of the geometry and of the underlying detector
-        
+
         :param config: dictionary with the configuration
         :return: itself
         """
-        version = int(config.get("poni_version", 1))
-
-        if "detector" in config:
-            self.detector = detectors.detector_factory(config["detector"],
-                                                       config.get("detector_config"))
-            if isinstance(self.detector, detectors.NexusDetector):
-                # increment the poni_version for Nexus detector as no further config is needed!
-                version = max(2, version)
-        else:
-            self.detector = detectors.Detector()
-        if version == 1:
-            # Handle former version of PONI-file
-            if self.detector.force_pixel and ("pixelsize1" in config) and ("pixelsize2" in config):
-                pixel1 = float(config["pixelsize1"])
-                pixel2 = float(config["pixelsize2"])
-                self.detector = self.detector.__class__(pixel1=pixel1, pixel2=pixel2)
-            else:
-                if "pixelsize1" in config:
-                    self.detector.pixel1 = float(config["pixelsize1"])
-                if "pixelsize2" in config:
-                    self.detector.pixel2 = float(config["pixelsize2"])
-
-        if "distance" in config:
-            self._dist = float(config["distance"])
-        if "poni1" in config:
-            self._poni1 = float(config["poni1"])
-        if "poni2" in config:
-            self._poni2 = float(config["poni2"])
-        if "rot1" in config:
-            self._rot1 = float(config["rot1"])
-        if "rot2" in config:
-            self._rot2 = float(config["rot2"])
-        if "rot3" in config:
-            self._rot3 = float(config["rot3"])
-        if "wavelength" in config:
-            self._wavelength = float(config["wavelength"])
-        if "splinefile" in config:
-            if config["splinefile"].lower() != "none":
-                self.detector.set_splineFile(config["splinefile"])
-        self.reset()
+        # TODO: ponifile should not be used here
+        #     if it was only used for IO, it would be better to remove
+        #     this function
+        poni = ponifile.PoniFile(config)
+        self._init_from_poni(poni)
         return self
 
     def save(self, filename):
@@ -1176,24 +1173,11 @@ class Geometry(object):
         """
         try:
             with open(filename, "a") as f:
-                f.write(("# Nota: C-Order, 1 refers to the Y axis,"
-                         " 2 to the X axis \n"))
-                f.write("# Calibration done at %s\n" % time.ctime())
-                f.write("poni_version: 2\n")
-                detector = self.detector
-                f.write("Detector: %s\n" % detector.name)
-                f.write("Detector_config: %s\n" % json.dumps(detector.get_config()))
-
-                f.write("Distance: %s\n" % self._dist)
-                f.write("Poni1: %s\n" % self._poni1)
-                f.write("Poni2: %s\n" % self._poni2)
-                f.write("Rot1: %s\n" % self._rot1)
-                f.write("Rot2: %s\n" % self._rot2)
-                f.write("Rot3: %s\n" % self._rot3)
-                if self._wavelength is not None:
-                    f.write("Wavelength: %s\n" % self._wavelength)
+                poni = ponifile.PoniFile(self)
+                poni.write(f)
         except IOError:
             logger.error("IOError while writing to file %s", filename)
+
     write = save
 
     @classmethod
@@ -1218,20 +1202,10 @@ class Geometry(object):
         :type filename: string
         :return: itself with updated parameters
         """
-        data = OrderedDict()
-        with open(filename) as opened_file:
-            for line in opened_file:
-                if line.startswith("#") or (":" not in line):
-                    continue
-                words = line.split(":", 1)
+        poni = ponifile.PoniFile(data=filename)
+        self._init_from_poni(poni)
+        return self
 
-                key = words[0].strip().lower()
-                try:
-                    value = words[1].strip()
-                except Exception as error:  # IGNORE:W0703:
-                    logger.error("Error %s with line: %s", error, line)
-                data[key] = value
-        return self.set_config(data)
     read = load
 
     def getPyFAI(self):
@@ -1270,7 +1244,7 @@ class Geometry(object):
                           self._rot1, self._rot2, self._rot3]
             self.chiDiscAtPi = True  # position of the discontinuity of chi in radians, pi by default
             self.reset()
-#            self._wavelength = None
+            # self._wavelength = None
             self._oversampling = None
             if self.splineFile:
                 self.detector.set_splineFile(self.splineFile)
@@ -1283,28 +1257,32 @@ class Geometry(object):
         :return: dict with parameters compatible with fit2D geometry
         """
         with self._sem:
-            cosTilt = cos(self._rot1) * cos(self._rot2)
-            sinTilt = sqrt(1 - cosTilt * cosTilt)
+            cos_tilt = cos(self._rot1) * cos(self._rot2)
+            sin_tilt = sqrt(1.0 - cos_tilt * cos_tilt)
+            tan_tilt = sin_tilt / cos_tilt
             # This is tilt plane rotation
-            if sinTilt != 0:
-                cosTpr = max(-1, min(1, -cos(self._rot2) * sin(self._rot1) / sinTilt))
-                sinTpr = sin(self._rot2) / sinTilt
-            else:  # undefined, does not seem to matter as not tilted
-                cosTpr = 1.0
-                sinTpr = 0.0
-            directDist = 1.0e3 * self._dist / cosTilt
-            tilt = degrees(arccos(cosTilt))
-            if sinTpr < 0:
-                tpr = -degrees(arccos(cosTpr))
-            else:
-                tpr = degrees(arccos(cosTpr))
+            if sin_tilt == 0:
+                # tilt plan rotation is undefined when there is no tilt!, does not matter
+                cos_tilt = 1.0
+                sin_tilt = 0.0
+                cos_tpr = 1.0
+                sin_tpr = 0.0
 
-            centerX = (self._poni2 + self._dist * sinTilt / cosTilt * cosTpr)\
-                / self.pixel2
-            if abs(tilt) < 1e-5:
+            else:
+                cos_tpr = max(-1.0, min(1.0, -cos(self._rot2) * sin(self._rot1) / sin_tilt))
+                sin_tpr = sin(self._rot2) / sin_tilt
+            directDist = 1.0e3 * self._dist / cos_tilt
+            tilt = degrees(arccos(cos_tilt))
+            if sin_tpr < 0:
+                tpr = -degrees(arccos(cos_tpr))
+            else:
+                tpr = degrees(arccos(cos_tpr))
+
+            centerX = (self._poni2 + self._dist * tan_tilt * cos_tpr) / self.pixel2
+            if abs(tilt) < 1e-5:  # in degree
                 centerY = (self._poni1) / self.pixel1
             else:
-                centerY = (self._poni1 + self._dist * sinTilt / cosTilt * sinTpr) / self.pixel1
+                centerY = (self._poni1 + self._dist * tan_tilt * sin_tpr) / self.pixel1
             out = self.detector.getFit2D()
             out["directDist"] = directDist
             out["centerX"] = centerX
@@ -1338,10 +1316,10 @@ class Geometry(object):
         """
         with self._sem:
             try:
-                cosTilt = cos(radians(tilt))
-                sinTilt = sin(radians(tilt))
-                cosTpr = cos(radians(tiltPlanRotation))
-                sinTpr = sin(radians(tiltPlanRotation))
+                cos_tilt = cos(radians(tilt))
+                sin_tilt = sin(radians(tilt))
+                cos_tpr = cos(radians(tiltPlanRotation))
+                sin_tpr = sin(radians(tiltPlanRotation))
             except AttributeError as error:
                 logger.error(("Got strange results with tilt=%s"
                               " and tiltPlanRotation=%s: %s") %
@@ -1353,20 +1331,18 @@ class Geometry(object):
                     self.detector.pixel2 = pixelX * 1.0e-6
             else:
                 self.detector.set_splineFile(splineFile)
-            self._dist = directDist * cosTilt * 1.0e-3
-            self._poni1 = centerY * self.pixel1\
-                - directDist * sinTilt * sinTpr * 1.0e-3
-            self._poni2 = centerX * self.pixel2\
-                - directDist * sinTilt * cosTpr * 1.0e-3
-            rot2 = numpy.arcsin(sinTilt * sinTpr)  # or pi-
-            rot1 = numpy.arccos(min(1.0, max(-1.0, (cosTilt / numpy.sqrt(1 - sinTpr * sinTpr * sinTilt * sinTilt)))))  # + or -
-            if cosTpr * sinTilt > 0:
+            self._dist = directDist * cos_tilt * 1.0e-3
+            self._poni1 = centerY * self.pixel1 - directDist * sin_tilt * sin_tpr * 1.0e-3
+            self._poni2 = centerX * self.pixel2 - directDist * sin_tilt * cos_tpr * 1.0e-3
+            rot2 = numpy.arcsin(sin_tilt * sin_tpr)  # or pi-
+            rot1 = numpy.arccos(min(1.0, max(-1.0, (cos_tilt / numpy.sqrt(1.0 - (sin_tpr * sin_tilt) ** 2)))))  # + or -
+            if cos_tpr * sin_tilt > 0:
                 rot1 = -rot1
-            assert abs(cosTilt - cos(rot1) * cos(rot2)) < 1e-6
-            if tilt == 0:
+            assert abs(cos_tilt - cos(rot1) * cos(rot2)) < 1e-6
+            if tilt == 0.0:
                 rot3 = 0
             else:
-                rot3 = numpy.arccos(min(1.0, max(-1.0, (cosTilt * cosTpr * sinTpr - cosTpr * sinTpr) / numpy.sqrt(1 - sinTpr * sinTpr * sinTilt * sinTilt))))  # + or -
+                rot3 = numpy.arccos(min(1.0, max(-1.0, (cos_tilt * cos_tpr * sin_tpr - cos_tpr * sin_tpr) / numpy.sqrt(10 - sin_tpr * sin_tpr * sin_tilt * sin_tilt))))  # + or -
                 rot3 = numpy.pi / 2.0 - rot3
             self._rot1 = rot1
             self._rot2 = rot2
@@ -1463,6 +1439,91 @@ class Geometry(object):
             logger.warning("Rotation conversion from pyFAI to SPD is not yet implemented")
         return res
 
+    def getImageD11(self):
+        """Export the current geometry in ImageD11 format.
+        Please refer to the documentation in doc/source/geometry_conversion.rst
+        for the orientation and units of those values.
+        
+        :return: an Ordered dict with those parameters:    
+            distance 294662.658 #in nm
+            o11 1
+            o12 0
+            o21 0
+            o22 -1
+            tilt_x 0.00000
+            tilt_y -0.013173
+            tilt_z 0.002378
+            wavelength 0.154
+            y-center 1016.328171
+            y-size 48.0815
+            z-center 984.924425
+            z-size 46.77648
+        """
+        f2d = self.getFit2D()
+        distance = f2d.get("directDist", 0) * 1e3  # mm -> µm
+        y_center = f2d.get("centerX", 0)  # in pixel
+        z_center = f2d.get("centerY", 0)  # in pixel
+
+        tilt_x = self.rot3
+        tilt_y = self.rot2
+        tilt_z = -self.rot1
+        out = OrderedDict([("distance", distance),
+                           ("o11", 1),
+                           ("o12", 0),
+                           ("o21", 0),
+                           ("o22", -1),
+                           ("tilt_x", tilt_x),
+                           ("tilt_y", tilt_y),
+                           ("tilt_z", tilt_z),
+                           ])
+        if self._wavelength:
+            out["wavelength"] = self.wavelength * 1e9  # nm
+        if y_center:
+            out["y-center"] = y_center
+        out["y-size"] = self.detector.pixel2 * 1e6  # µm
+        if z_center:
+            out["z-center"] = z_center
+        out["z-size"] = self.detector.pixel1 * 1e6  # µm
+        return out
+
+    def setImageD11(self, param):
+        """Set the geometry from the parameter set which contains distance, 
+        o11, o12, o21, o22, tilt_x, tilt_y tilt_z, wavelength, y-center, y-size, 
+        z-center and z-size. 
+        Please refer to the documentation in doc/source/geometry_conversion.rst
+        for the orientation and units of those values.
+        
+        :param param: dict with the values to set.
+        """
+        o11 = param.get("o11")
+        if o11 is not None:
+            assert o11 == 1, "Only canonical orientation is supported"
+        o12 = param.get("o12")
+        if o12 is not None:
+            assert o12 == 0, "Only canonical orientation is supported"
+        o21 = param.get("o21")
+        if o21 is not None:
+            assert o21 == 0, "Only canonical orientation is supported"
+        o22 = param.get("o22")
+        if o22 is not None:
+            assert o22 == -1, "Only canonical orientation is supported"
+
+        self.rot3 = param.get("tilt_x", 0.0)
+        self.rot2 = param.get("tilt_y", 0.0)
+        self.rot1 = -param.get("tilt_z", 0.0)
+        distance = param.get("distance", 0.0) * 1e-6  # ->m
+        self.dist = distance * cos(self.rot2) * cos(self.rot1)
+        pixel_v = param.get("z-size", 0.0) * 1e-6
+        pixel_h = param.get("y-size", 0.0) * 1e-6
+        self.poni1 = -distance * sin(self.rot2) + pixel_v * param.get("z-center", 0.0)
+        self.poni2 = +distance * cos(self.rot2) * sin(self.rot1) + pixel_h * param.get("y-center", 0.0)
+        self.detector = detectors.Detector(pixel1=pixel_v, pixel2=pixel_h)
+        wl = param.get("wavelength")
+        if wl:
+            self.wavelength = wl * 1e-9
+        self.reset()
+        return self
+
     def set_param(self, param):
         """set the geometry from a 6-tuple with dist, poni1, poni2, rot1, rot2,
         rot3
@@ -1746,7 +1807,7 @@ class Geometry(object):
             shape = self.detector.max_shape
         try:
             ttha = self.__getattribute__(dim1_unit.center)(shape)
-        except:
+        except Exception:
             raise RuntimeError("in pyFAI.Geometry.calcfrom1d: " +
                                str(dim1_unit) + " not (yet?) Implemented")
         calcimage = numpy.interp(ttha.ravel(), tth, I)
@@ -1800,11 +1861,10 @@ class Geometry(object):
             shape = self.detector.max_shape
         try:
             ttha = self.__getattribute__(dim1_unit.center)(shape)
-
-        except:
-            raise RuntimeError("in pyFAI.Geometry.calcfrom1d: " +
+        except Exception:
+            raise RuntimeError("in pyFAI.Geometry.calcfrom2d: " +
                                str(dim1_unit) + " not (yet?) Implemented")
-        chia = self.chia
+        chia = self.chiArray(shape)
 
         built_mask = numpy.ones(shape, dtype=numpy.int8)
         empty_data = numpy.zeros(shape, dtype=numpy.float32)
@@ -1879,7 +1939,7 @@ class Geometry(object):
         new.param = new_param
         cached = {}
         memo[id(self._cached_array)] = cached
-        for key, old_value in self._cached_array.items():
+        for key, old_value in self._cached_array.copy().items():
             if "copy" in dir(old_value):
                 new_value = old_value.copy()
                 memo[id(old_value)] = new_value
@@ -2065,9 +2125,6 @@ class Geometry(object):
         self._cached_array["q_corner"] = q_corner
 
     def get_wavelength(self):
-        if self._wavelength is None:
-            raise RuntimeWarning("Using wavelength without having defined"
-                                 " it previously ... excepted to fail !")
         return self._wavelength
 
     wavelength = property(get_wavelength, set_wavelength)
@@ -2080,6 +2137,7 @@ class Geometry(object):
 
     def del_ttha(self):
         self._cached_array["2th_center"] = None
+
     ttha = property(get_ttha, set_ttha, del_ttha, "2theta array in cache")
 
     def get_chia(self):
@@ -2090,6 +2148,7 @@ class Geometry(object):
 
     def del_chia(self):
         self._cached_array["chi_center"] = None
+
     chia = property(get_chia, set_chia, del_chia, "chi array in cache")
 
     def get_dssa(self):
@@ -2113,6 +2172,7 @@ class Geometry(object):
 
     def del_qa(self):
         self._cached_array["q_center"] = None
+
     qa = property(get_qa, set_qa, del_qa, "Q array in cache")
 
     def get_ra(self):
@@ -2123,6 +2183,7 @@ class Geometry(object):
 
     def del_ra(self):
         self.self._cached_array["r_center"] = None
+
     ra = property(get_ra, set_ra, del_ra, "R array in cache")
 
     def get_pixel1(self):
@@ -2218,7 +2279,7 @@ class Geometry(object):
 
     def __getstate__(self):
         """Helper function for pickling geometry
-        
+
         :return: the state of the object
         """
 
@@ -2231,7 +2292,7 @@ class Geometry(object):
 
     def __setstate__(self, state):
         """Helper function for unpickling geometry
-        
+
         :param state: the state of the object
         """
         for statekey, statevalue in state.items():
