@@ -34,7 +34,7 @@ __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "17/07/2018"
+__date__ = "06/05/2019"
 
 import unittest
 import os
@@ -53,7 +53,7 @@ from ..azimuthalIntegrator import AzimuthalIntegrator
 from ..detectors import Detector
 if logger.getEffectiveLevel() <= logging.DEBUG:
     import pylab
-from pyFAI import units
+from pyFAI import units, detector_factory
 from ..utils import mathutil
 from ..third_party import six
 from pyFAI.utils.decorators import depreclog
@@ -158,12 +158,11 @@ class TestAzimHalfFrelon(unittest.TestCase):
         """
         Compare numpy histogram with results of fit2d
         """
-        # logger.info(self.ai.__repr__())
-        with utilstest.TestLogging(logger=depreclog, warning=1):
-            # Filter deprecated warning
-            tth, I = self.ai.xrpd_numpy(self.data,
-                                        len(self.fit2d), self.tmpfiles["numpy"],
-                                        correctSolidAngle=False)
+        tth, I = self.ai.integrate1d(self.data,
+                                     len(self.fit2d),
+                                     filename=self.tmpfiles["numpy"],
+                                     correctSolidAngle=False,
+                                     unit="2th_deg")
         rwp = mathutil.rwp((tth, I), self.fit2d.T)
         logger.info("Rwp numpy/fit2d = %.3f", rwp)
         if logger.getEffectiveLevel() == logging.DEBUG:
@@ -184,11 +183,12 @@ class TestAzimHalfFrelon(unittest.TestCase):
         """
         Compare cython histogram with results of fit2d
         """
-        with utilstest.TestLogging(logger=depreclog, warning=1):
-            # Filter deprecated warning
-            tth, I = self.ai.xrpd_cython(self.data,
-                                         len(self.fit2d), self.tmpfiles["cython"],
-                                         correctSolidAngle=False, pixelSize=None)
+        tth, I = self.ai.integrate1d(self.data,
+                                     len(self.fit2d),
+                                     filename=self.tmpfiles["cython"],
+                                     correctSolidAngle=False,
+                                     unit='2th_deg',
+                                     method="cython")
         # logger.info(tth)
         # logger.info(I)
         rwp = mathutil.rwp((tth, I), self.fit2d.T)
@@ -217,16 +217,14 @@ class TestAzimHalfFrelon(unittest.TestCase):
         t0 = time.time()
         logger.info("in test_cythonSP_vs_fit2d Before SP")
 
-        with utilstest.TestLogging(logger=depreclog, warning=1):
-            # Filter deprecated warning
-            tth, I = self.ai.xrpd_splitPixel(self.data,
-                                             len(self.fit2d),
-                                             self.tmpfiles["cythonSP"],
-                                             correctSolidAngle=False)
+        tth, I = self.ai.integrate1d(self.data,
+                                     len(self.fit2d),
+                                     filename=self.tmpfiles["cythonSP"],
+                                     method="splitpixel",
+                                     correctSolidAngle=False,
+                                     unit="2th_deg")
         logger.info("in test_cythonSP_vs_fit2d Before")
         t1 = time.time() - t0
-        # logger.info(tth)
-        # logger.info(I)
         rwp = mathutil.rwp((tth, I), self.fit2d.T)
         logger.info("Rwp cythonSP(t=%.3fs)/fit2d = %.3f", t1, rwp)
         if logger.getEffectiveLevel() == logging.DEBUG:
@@ -247,18 +245,22 @@ class TestAzimHalfFrelon(unittest.TestCase):
         """
         Compare cython histogram with numpy histogram
         """
-        with utilstest.TestLogging(logger=depreclog, warning=3):
-            # Filter deprecated warning
-            tth_np, I_np = self.ai.xrpd_numpy(self.__class__.data,
-                                              len(self.fit2d),
-                                              correctSolidAngle=False)
-            tth_cy, I_cy = self.ai.xrpd_cython(self.__class__.data,
-                                               len(self.fit2d),
-                                               correctSolidAngle=False)
-            logger.info("before xrpd_splitPixel")
-            tth_sp, I_sp = self.ai.xrpd_splitPixel(self.__class__.data,
-                                                   len(self.fit2d),
-                                                   correctSolidAngle=False)
+        tth_np, I_np = self.ai.integrate1d(self.__class__.data,
+                                           len(self.fit2d),
+                                           correctSolidAngle=False,
+                                           unit="2th_deg",
+                                           method="numpy")
+        tth_cy, I_cy = self.ai.integrate1d(self.__class__.data,
+                                           len(self.fit2d),
+                                           correctSolidAngle=False,
+                                           unit="2th_deg",
+                                           method="cython")
+        logger.info("before xrpd_splitPixel")
+        tth_sp, I_sp = self.ai.integrate1d(self.__class__.data,
+                                           len(self.fit2d),
+                                           correctSolidAngle=False,
+                                           unit="2th_deg",
+                                           method="splitpixel")
         logger.info("After xrpd_splitPixel")
         rwp = mathutil.rwp((tth_cy, I_cy), (tth_np, I_np))
         logger.info("Rwp = %.3f", rwp)
@@ -288,14 +290,14 @@ class TestAzimHalfFrelon(unittest.TestCase):
     @unittest.skipIf(UtilsTest.opencl is False, "User request to skip OpenCL tests")
     @unittest.skipIf(UtilsTest.low_mem, "test using >100Mb")
     def test_medfilt1d(self):
-        ref = self.ai.medfilt1d(self.data, 1000, unit="2th_deg", method="csr")
-        ocl = self.ai.medfilt1d(self.data, 1000, unit="2th_deg", method="ocl_csr")
+        ref = self.ai.medfilt1d(self.data, 1000, unit="2th_deg", method="bbox_csr")
+        ocl = self.ai.medfilt1d(self.data, 1000, unit="2th_deg", method="bbox_ocl_csr")
         rwp = mathutil.rwp(ref, ocl)
         logger.info("test_medfilt1d median Rwp = %.3f", rwp)
         self.assertLess(rwp, 1, "Rwp medfilt1d Numpy/OpenCL: %.3f" % rwp)
 
-        ref = self.ai.medfilt1d(self.data, 1000, unit="2th_deg", method="csr", percentile=(20, 80))
-        ocl = self.ai.medfilt1d(self.data, 1000, unit="2th_deg", method="ocl_csr", percentile=(20, 80))
+        ref = self.ai.medfilt1d(self.data, 1000, unit="2th_deg", method="bbox_csr", percentile=(20, 80))
+        ocl = self.ai.medfilt1d(self.data, 1000, unit="2th_deg", method="bbox_ocl_csr", percentile=(20, 80))
         rwp = mathutil.rwp(ref, ocl)
         logger.info("test_medfilt1d trimmed-mean Rwp = %.3f", rwp)
         self.assertLess(rwp, 3, "Rwp trimmed-mean Numpy/OpenCL: %.3f" % rwp)
@@ -327,11 +329,8 @@ class TestFlatimage(unittest.TestCase):
         det = Detector(1e-4, 1e-4, max_shape=shape)
         ai = AzimuthalIntegrator(0.1, 1e-2, 1e-2, detector=det)
 
-        with utilstest.TestLogging(logger=depreclog, warning=1):
-            # Filter deprecated warning
-            I = ai.xrpd2_splitPixel(data, 256, 2256, correctSolidAngle=False, dummy=-1.0)[0]
-        # I = ai.xrpd2(data, 2048, 2048, correctSolidAngle=False, dummy= -1.0)
-
+        I = ai.integrate2d(data, 256, 2256, correctSolidAngle=False, dummy=-1.0,
+                           method='splitpixel', unit='2th_deg')[0]
         if logger.getEffectiveLevel() == logging.DEBUG:
             logging.info("Plotting results")
             fig = pylab.figure()
@@ -349,10 +348,8 @@ class TestFlatimage(unittest.TestCase):
         data = numpy.ones(shape, dtype="float64")
         det = Detector(1e-4, 1e-4, max_shape=shape)
         ai = AzimuthalIntegrator(0.1, 1e-2, 1e-2, detector=det)
-
-        with utilstest.TestLogging(logger=depreclog, warning=1):
-            # Filter deprecated warning
-            I = ai.xrpd2_splitBBox(data, 256, 256, correctSolidAngle=False, dummy=-1.0)[0]
+        I = ai.integrate2d(data, 256, 256, correctSolidAngle=False, dummy=-1.0,
+                           unit="2th_deg", method='splitbbox')[0]
 
         if logger.getEffectiveLevel() == logging.DEBUG:
             logging.info("Plotting results")
@@ -525,6 +522,48 @@ class TestSetter(unittest.TestCase):
         self.assertTrue(abs(self.ai.darkcurrent - 0.5 * (self.rnd1 + self.rnd2)).max() == 0, "Dark array is OK")
 
 
+class TestIntergrationNextGeneration(unittest.TestCase):
+
+    def test_histo(self):
+        det = detector_factory("Pilatus100k")
+        data = numpy.random.random(det.shape)
+        ai = AzimuthalIntegrator(detector=det, wavelength=1e-10)
+
+        method = ("no", "histogram", "python")
+        python = ai._integrate1d_ng(data, 100, method=method, error_model="poisson")
+        self.assertEqual(python.compute_engine, "pyFAI.engines.histogram_engine.histogram1d_engine")
+        self.assertEqual(str(python.unit), "q_nm^-1")
+
+        method = ("no", "histogram", "cython")
+        cython = ai._integrate1d_ng(data, 100, method=method, error_model="poisson")
+        self.assertEqual(cython.compute_engine, "pyFAI.ext.histogram.histogram1d_engine")
+        self.assertEqual(str(cython.unit), "q_nm^-1")
+        self.assertTrue(numpy.allclose(cython.radial, python.radial), "cython position are the same")
+        self.assertTrue(numpy.allclose(cython.intensity, python.intensity), "cython intensities are the same")
+        self.assertTrue(numpy.allclose(cython.sigma, python.sigma), "cython errors are the same")
+        self.assertTrue(numpy.allclose(cython.sum_signal, python.sum_signal), "cython sum_signal are the same")
+        self.assertTrue(numpy.allclose(cython.sum_variance, python.sum_variance), "cython sum_variance are the same")
+        self.assertTrue(numpy.allclose(cython.sum_normalization, python.sum_normalization), "cython sum_normalization are the same")
+        self.assertTrue(numpy.allclose(cython.count, python.count), "cython count are the same")
+
+        method = ("no", "histogram", "opencl")
+        actual_method = ai._normalize_method(method=method, dim=1, default=ai.DEFAULT_METHOD_1D).method[1:4]
+        if actual_method != method:
+            reason = "Skipping TestIntergrationNextGeneration.test_histo as OpenCL method not available"
+            self.skipTest(reason)
+        opencl = ai._integrate1d_ng(data, 100, method=method, error_model="poisson")
+        self.assertEqual(opencl.compute_engine, "pyFAI.opencl.azim_hist.OCL_Histogram1d")
+        self.assertEqual(str(opencl.unit), "q_nm^-1")
+
+        self.assertTrue(numpy.allclose(opencl.radial, python.radial), "opencl position are the same")
+        self.assertTrue(numpy.allclose(opencl.intensity, python.intensity), "opencl intensities are the same")
+        self.assertTrue(numpy.allclose(opencl.sigma, python.sigma), "opencl errors are the same")
+        self.assertTrue(numpy.allclose(opencl.sum_signal.sum(axis=-1), python.sum_signal), "opencl sum_signal are the same")
+        self.assertTrue(numpy.allclose(opencl.sum_variance.sum(axis=-1), python.sum_variance), "opencl sum_variance are the same")
+        self.assertTrue(numpy.allclose(opencl.sum_normalization.sum(axis=-1), python.sum_normalization), "opencl sum_normalization are the same")
+        self.assertTrue(numpy.allclose(opencl.count, python.count), "opencl count are the same")
+
+
 def suite():
     loader = unittest.defaultTestLoader.loadTestsFromTestCase
     testsuite = unittest.TestSuite()
@@ -534,6 +573,7 @@ def suite():
     # Consumes a lot of memory
     # testsuite.addTest(loader(TestAzimPilatus))
     testsuite.addTest(loader(TestSaxs))
+    testsuite.addTest(loader(TestIntergrationNextGeneration))
     return testsuite
 
 
